@@ -29,7 +29,6 @@ import (
 	"k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
-	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/runtime/schema"
 	"k8s.io/apimachinery/pkg/types"
 	lifecycleapi "kubevirt.io/controller-lifecycle-operator-sdk/pkg/sdk/api"
@@ -46,24 +45,14 @@ import (
 	ssp "kubevirt.io/ssp-operator/api/v1beta1"
 	"kubevirt.io/ssp-operator/internal/common"
 	"kubevirt.io/ssp-operator/internal/operands"
-	common_templates "kubevirt.io/ssp-operator/internal/operands/common-templates"
-	"kubevirt.io/ssp-operator/internal/operands/metrics"
-	node_labeller "kubevirt.io/ssp-operator/internal/operands/node-labeller"
-	template_validator "kubevirt.io/ssp-operator/internal/operands/template-validator"
 )
 
 const (
-	finalizerName          = "ssp.kubevirt.io/finalizer"
-	oldFinalizerName       = "finalize.ssp.kubevirt.io"
-	defaultOperatorVersion = "devel"
-)
+	finalizerName    = "ssp.kubevirt.io/finalizer"
+	oldFinalizerName = "finalize.ssp.kubevirt.io"
 
-var sspOperands = []operands.Operand{
-	metrics.GetOperand(),
-	template_validator.GetOperand(),
-	common_templates.GetOperand(),
-	node_labeller.GetOperand(),
-}
+	templateBundleDir = "data/common-templates-bundle/"
+)
 
 // List of legacy CRDs and their corresponding kinds
 var kvsspCRDs = map[string]string{
@@ -73,16 +62,25 @@ var kvsspCRDs = map[string]string{
 	"kubevirtnodelabellerbundles.ssp.kubevirt.io":    "KubevirtNodeLabellerBundle",
 }
 
-// SSPReconciler reconciles a SSP object
-type SSPReconciler struct {
-	client.Client
-	Log logr.Logger
-
-	LastSspSpec      ssp.SSPSpec
-	SubresourceCache common.VersionCache
+// sspReconciler reconciles a SSP object
+type sspReconciler struct {
+	client           client.Client
+	log              logr.Logger
+	operands         []operands.Operand
+	lastSspSpec      ssp.SSPSpec
+	subresourceCache common.VersionCache
 }
 
-var _ reconcile.Reconciler = &SSPReconciler{}
+func NewSspReconciler(client client.Client, operands []operands.Operand) *sspReconciler {
+	return &sspReconciler{
+		client:           client,
+		log:              ctrl.Log.WithName("controllers").WithName("SSP"),
+		operands:         operands,
+		subresourceCache: common.VersionCache{},
+	}
+}
+
+var _ reconcile.Reconciler = &sspReconciler{}
 
 // +kubebuilder:rbac:groups=ssp.kubevirt.io,resources=ssps,verbs=get;list;watch;create;update;patch;delete
 // +kubebuilder:rbac:groups=ssp.kubevirt.io,resources=ssps/status,verbs=get;update;patch
@@ -93,13 +91,31 @@ var _ reconcile.Reconciler = &SSPReconciler{}
 // +kubebuilder:rbac:groups=ssp.kubevirt.io,resources=kubevirtnodelabellerbundles,verbs=get;list;watch;create;update;patch;delete
 // +kubebuilder:rbac:groups=ssp.kubevirt.io,resources=kubevirttemplatevalidators,verbs=get;list;watch;create;update;patch;delete
 
-func (r *SSPReconciler) Reconcile(ctx context.Context, req ctrl.Request) (ctrl.Result, error) {
-	reqLogger := r.Log.WithValues("ssp", req.NamespacedName)
+func (r *sspReconciler) setupController(mgr ctrl.Manager) error {
+	builder := ctrl.NewControllerManagedBy(mgr)
+	watchSspResource(builder)
+	watchClusterResources(builder, r.operands)
+	watchNamespacedResources(builder, r.operands)
+	return builder.Complete(r)
+}
+
+// Reconcile is part of the main kubernetes reconciliation loop which aims to
+// move the current state of the cluster closer to the desired state.
+//
+// For more details, check Reconcile and its Result here:
+// - https://pkg.go.dev/sigs.k8s.io/controller-runtime@v0.7.0/pkg/reconcile
+func (r *sspReconciler) Reconcile(ctx context.Context, req ctrl.Request) (res ctrl.Result, err error) {
+	defer func() {
+		if err != nil {
+			common.SSPOperatorReconcilingProperly.Set(0)
+		}
+	}()
+	reqLogger := r.log.WithValues("ssp", req.NamespacedName)
 	reqLogger.V(1).Info("Starting reconciliation...")
 
 	// Fetch the SSP instance
 	instance := &ssp.SSP{}
-	err := r.Get(ctx, req.NamespacedName, instance)
+	err = r.client.Get(ctx, req.NamespacedName, instance)
 	if err != nil {
 		if errors.IsNotFound(err) {
 			// Request object not found, could have been deleted after reconcile request.
@@ -115,11 +131,11 @@ func (r *SSPReconciler) Reconcile(ctx context.Context, req ctrl.Request) (ctrl.R
 
 	sspRequest := &common.Request{
 		Request:      req,
-		Client:       r,
+		Client:       r.client,
 		Context:      ctx,
 		Instance:     instance,
 		Logger:       reqLogger,
-		VersionCache: r.SubresourceCache,
+		VersionCache: r.subresourceCache,
 	}
 
 	if !isInitialized(sspRequest.Instance) {
@@ -135,7 +151,7 @@ func (r *SSPReconciler) Reconcile(ctx context.Context, req ctrl.Request) (ctrl.R
 	}
 
 	if isBeingDeleted(sspRequest.Instance) {
-		err := cleanup(sspRequest)
+		err := r.cleanup(sspRequest)
 		if err != nil {
 			return ctrl.Result{}, err
 		}
@@ -150,7 +166,7 @@ func (r *SSPReconciler) Reconcile(ctx context.Context, req ctrl.Request) (ctrl.R
 		reqLogger.Info(fmt.Sprintf("Pausing SSP operator on resource: %v/%v", instance.Namespace, instance.Name))
 		instance.Status.Paused = true
 		instance.Status.ObservedGeneration = instance.Generation
-		err := r.Status().Update(ctx, instance)
+		err := r.client.Status().Update(ctx, instance)
 		return ctrl.Result{}, err
 	}
 
@@ -162,36 +178,38 @@ func (r *SSPReconciler) Reconcile(ctx context.Context, req ctrl.Request) (ctrl.R
 	sspRequest.Logger.V(1).Info("CR status updated")
 
 	sspRequest.Logger.V(1).Info("Reconciling operands...")
-	statuses, err := reconcileOperands(sspRequest)
+	reconcileResults, err := r.reconcileOperands(sspRequest)
 	if err != nil {
 		return handleError(sspRequest, err)
 	}
 	sspRequest.Logger.V(1).Info("Operands reconciled")
 
 	sspRequest.Logger.V(1).Info("Updating CR status post reconciliation...")
-	err = updateStatus(sspRequest, statuses)
+	err = updateStatus(sspRequest, reconcileResults)
 	if err != nil {
 		return ctrl.Result{}, err
 	}
 	sspRequest.Logger.V(1).Info("CR status updated")
 
+	if sspRequest.Instance.Status.Phase == lifecycleapi.PhaseDeployed {
+		common.SSPOperatorReconcilingProperly.Set(1)
+	} else {
+		common.SSPOperatorReconcilingProperly.Set(0)
+	}
+
 	return ctrl.Result{}, nil
 }
 
-func (r *SSPReconciler) clearCacheIfNeeded(sspObj *ssp.SSP) {
-	if !reflect.DeepEqual(r.LastSspSpec, sspObj.Spec) {
-		r.SubresourceCache = common.VersionCache{}
-		r.LastSspSpec = sspObj.Spec
+func (r *sspReconciler) clearCacheIfNeeded(sspObj *ssp.SSP) {
+	if !reflect.DeepEqual(r.lastSspSpec, sspObj.Spec) {
+		r.subresourceCache = common.VersionCache{}
+		r.lastSspSpec = sspObj.Spec
 	}
 }
 
-func (r *SSPReconciler) clearCache() {
-	r.LastSspSpec = ssp.SSPSpec{}
-	r.SubresourceCache = common.VersionCache{}
-}
-
-func getOperatorVersion() string {
-	return common.EnvOrDefault(common.OperatorVersionKey, defaultOperatorVersion)
+func (r *sspReconciler) clearCache() {
+	r.lastSspSpec = ssp.SSPSpec{}
+	r.subresourceCache = common.VersionCache{}
 }
 
 func isPaused(object metav1.Object) bool {
@@ -251,22 +269,55 @@ func updateSspResource(request *common.Request) error {
 	return request.Client.Status().Update(request.Context, request.Instance)
 }
 
-func cleanup(request *common.Request) error {
+func (r *sspReconciler) cleanup(request *common.Request) error {
 	if controllerutil.ContainsFinalizer(request.Instance, finalizerName) ||
 		controllerutil.ContainsFinalizer(request.Instance, oldFinalizerName) {
+		sspStatus := &request.Instance.Status
+		sspStatus.Phase = lifecycleapi.PhaseDeleting
+		sspStatus.ObservedGeneration = request.Instance.Generation
+		conditionsv1.SetStatusCondition(&sspStatus.Conditions, conditionsv1.Condition{
+			Type:    conditionsv1.ConditionAvailable,
+			Status:  v1.ConditionFalse,
+			Reason:  "available",
+			Message: "Deleting SSP resources",
+		})
+		conditionsv1.SetStatusCondition(&sspStatus.Conditions, conditionsv1.Condition{
+			Type:    conditionsv1.ConditionProgressing,
+			Status:  v1.ConditionTrue,
+			Reason:  "progressing",
+			Message: "Deleting SSP resources",
+		})
+		conditionsv1.SetStatusCondition(&sspStatus.Conditions, conditionsv1.Condition{
+			Type:    conditionsv1.ConditionDegraded,
+			Status:  v1.ConditionTrue,
+			Reason:  "degraded",
+			Message: "Deleting SSP resources",
+		})
 
-		request.Instance.Status.Phase = lifecycleapi.PhaseDeleting
-		request.Instance.Status.ObservedGeneration = request.Instance.Generation
 		err := request.Client.Status().Update(request.Context, request.Instance)
 		if err != nil {
 			return err
 		}
-		for _, operand := range sspOperands {
-			err = operand.Cleanup(request)
+
+		pendingCount := 0
+		for _, operand := range r.operands {
+			cleanupResults, err := operand.Cleanup(request)
 			if err != nil {
 				return err
 			}
+
+			for _, result := range cleanupResults {
+				if !result.Deleted {
+					pendingCount += 1
+				}
+			}
 		}
+
+		if pendingCount > 0 {
+			// Will retry cleanup on next reconciliation iteration
+			return nil
+		}
+
 		controllerutil.RemoveFinalizer(request.Instance, finalizerName)
 		controllerutil.RemoveFinalizer(request.Instance, oldFinalizerName)
 		err = request.Client.Update(request.Context, request.Instance)
@@ -338,7 +389,7 @@ func listExistingCRDKinds(sspRequest *common.Request) []string {
 	return foundKinds
 }
 
-func reconcileOperands(sspRequest *common.Request) ([]common.ResourceStatus, error) {
+func (r *sspReconciler) reconcileOperands(sspRequest *common.Request) ([]common.ReconcileResult, error) {
 	kinds := listExistingCRDKinds(sspRequest)
 
 	// Mark existing CRs as paused
@@ -348,22 +399,22 @@ func reconcileOperands(sspRequest *common.Request) ([]common.ResourceStatus, err
 	}
 
 	// Reconcile all operands
-	allStatuses := make([]common.ResourceStatus, 0, len(sspOperands))
-	for _, operand := range sspOperands {
+	allReconcileResults := make([]common.ReconcileResult, 0, len(r.operands))
+	for _, operand := range r.operands {
 		sspRequest.Logger.V(1).Info(fmt.Sprintf("Reconciling operand: %s", operand.Name()))
-		statuses, err := operand.Reconcile(sspRequest)
+		reconcileResults, err := operand.Reconcile(sspRequest)
 		if err != nil {
 			sspRequest.Logger.V(1).Info(fmt.Sprintf("Operand reconciliation failed: %s", err.Error()))
 			return nil, err
 		}
-		allStatuses = append(allStatuses, statuses...)
+		allReconcileResults = append(allReconcileResults, reconcileResults...)
 	}
 
-	return allStatuses, nil
+	return allReconcileResults, nil
 }
 
 func preUpdateStatus(request *common.Request) error {
-	operatorVersion := getOperatorVersion()
+	operatorVersion := common.GetOperatorVersion()
 
 	sspStatus := &request.Instance.Status
 	sspStatus.Phase = lifecycleapi.PhaseDeploying
@@ -407,19 +458,19 @@ func preUpdateStatus(request *common.Request) error {
 	return request.Client.Status().Update(request.Context, request.Instance)
 }
 
-func updateStatus(request *common.Request, statuses []common.ResourceStatus) error {
-	notAvailable := make([]common.ResourceStatus, 0, len(statuses))
-	progressing := make([]common.ResourceStatus, 0, len(statuses))
-	degraded := make([]common.ResourceStatus, 0, len(statuses))
-	for _, status := range statuses {
-		if status.NotAvailable != nil {
-			notAvailable = append(notAvailable, status)
+func updateStatus(request *common.Request, reconcileResults []common.ReconcileResult) error {
+	notAvailable := make([]common.ReconcileResult, 0, len(reconcileResults))
+	progressing := make([]common.ReconcileResult, 0, len(reconcileResults))
+	degraded := make([]common.ReconcileResult, 0, len(reconcileResults))
+	for _, reconcileResult := range reconcileResults {
+		if reconcileResult.Status.NotAvailable != nil {
+			notAvailable = append(notAvailable, reconcileResult)
 		}
-		if status.Progressing != nil {
-			progressing = append(progressing, status)
+		if reconcileResult.Status.Progressing != nil {
+			progressing = append(progressing, reconcileResult)
 		}
-		if status.Degraded != nil {
-			degraded = append(degraded, status)
+		if reconcileResult.Status.Degraded != nil {
+			degraded = append(degraded, reconcileResult)
 		}
 	}
 
@@ -433,12 +484,12 @@ func updateStatus(request *common.Request, statuses []common.ResourceStatus) err
 			Message: "All SSP resources are available",
 		})
 	case 1:
-		status := notAvailable[0]
+		reconcileResult := notAvailable[0]
 		conditionsv1.SetStatusCondition(&sspStatus.Conditions, conditionsv1.Condition{
 			Type:    conditionsv1.ConditionAvailable,
 			Status:  v1.ConditionFalse,
 			Reason:  "available",
-			Message: prefixResourceTypeAndName(*status.NotAvailable, status.Resource),
+			Message: prefixResourceTypeAndName(*reconcileResult.Status.NotAvailable, reconcileResult.Resource),
 		})
 	default:
 		conditionsv1.SetStatusCondition(&sspStatus.Conditions, conditionsv1.Condition{
@@ -458,12 +509,12 @@ func updateStatus(request *common.Request, statuses []common.ResourceStatus) err
 			Message: "No SSP resources are progressing",
 		})
 	case 1:
-		status := progressing[0]
+		reconcileResult := progressing[0]
 		conditionsv1.SetStatusCondition(&sspStatus.Conditions, conditionsv1.Condition{
 			Type:    conditionsv1.ConditionProgressing,
 			Status:  v1.ConditionTrue,
 			Reason:  "progressing",
-			Message: prefixResourceTypeAndName(*status.Progressing, status.Resource),
+			Message: prefixResourceTypeAndName(*reconcileResult.Status.Progressing, reconcileResult.Resource),
 		})
 	default:
 		conditionsv1.SetStatusCondition(&sspStatus.Conditions, conditionsv1.Condition{
@@ -483,12 +534,12 @@ func updateStatus(request *common.Request, statuses []common.ResourceStatus) err
 			Message: "No SSP resources are degraded",
 		})
 	case 1:
-		status := degraded[0]
+		reconcileResult := degraded[0]
 		conditionsv1.SetStatusCondition(&sspStatus.Conditions, conditionsv1.Condition{
 			Type:    conditionsv1.ConditionDegraded,
 			Status:  v1.ConditionTrue,
 			Reason:  "degraded",
-			Message: prefixResourceTypeAndName(*status.Degraded, status.Resource),
+			Message: prefixResourceTypeAndName(*reconcileResult.Status.Degraded, reconcileResult.Resource),
 		})
 	default:
 		conditionsv1.SetStatusCondition(&sspStatus.Conditions, conditionsv1.Condition{
@@ -502,7 +553,7 @@ func updateStatus(request *common.Request, statuses []common.ResourceStatus) err
 	sspStatus.ObservedGeneration = request.Instance.Generation
 	if len(notAvailable) == 0 && len(progressing) == 0 && len(degraded) == 0 {
 		sspStatus.Phase = lifecycleapi.PhaseDeployed
-		sspStatus.ObservedVersion = getOperatorVersion()
+		sspStatus.ObservedVersion = common.GetOperatorVersion()
 	} else {
 		sspStatus.Phase = lifecycleapi.PhaseDeploying
 	}
@@ -559,16 +610,6 @@ func handleError(request *common.Request, errParam error) (ctrl.Result, error) {
 	return ctrl.Result{}, errParam
 }
 
-func (r *SSPReconciler) SetupWithManager(mgr ctrl.Manager) error {
-	r.SubresourceCache = common.VersionCache{}
-
-	builder := ctrl.NewControllerManagedBy(mgr)
-	watchSspResource(builder)
-	watchClusterResources(builder)
-	watchNamespacedResources(builder)
-	return builder.Complete(r)
-}
-
 func watchSspResource(bldr *ctrl.Builder) {
 	// Predicate is used to only reconcile on these changes to the SSP resource:
 	// - any change in spec - checked with generation
@@ -592,29 +633,31 @@ func watchSspResource(bldr *ctrl.Builder) {
 	bldr.For(&ssp.SSP{}, builder.WithPredicates(pred))
 }
 
-func watchNamespacedResources(builder *ctrl.Builder) {
+func watchNamespacedResources(builder *ctrl.Builder, sspOperands []operands.Operand) {
 	watchResources(builder,
 		&handler.EnqueueRequestForOwner{
 			IsController: true,
 			OwnerType:    &ssp.SSP{},
 		},
+		sspOperands,
 		operands.Operand.WatchTypes,
 	)
 }
 
-func watchClusterResources(builder *ctrl.Builder) {
+func watchClusterResources(builder *ctrl.Builder, sspOperands []operands.Operand) {
 	watchResources(builder,
 		&libhandler.EnqueueRequestForAnnotation{
 			Type: schema.GroupKind{
-				Group: "ssp.kubevirt.io",
+				Group: ssp.GroupVersion.Group,
 				Kind:  "SSP",
 			},
 		},
+		sspOperands,
 		operands.Operand.WatchClusterTypes,
 	)
 }
 
-func watchResources(builder *ctrl.Builder, handler handler.EventHandler, watchTypesFunc func(operands.Operand) []client.Object) {
+func watchResources(builder *ctrl.Builder, handler handler.EventHandler, sspOperands []operands.Operand, watchTypesFunc func(operands.Operand) []client.Object) {
 	watchedTypes := make(map[reflect.Type]struct{})
 	for _, operand := range sspOperands {
 		for _, t := range watchTypesFunc(operand) {
@@ -627,13 +670,3 @@ func watchResources(builder *ctrl.Builder, handler handler.EventHandler, watchTy
 		}
 	}
 }
-
-func InitScheme(scheme *runtime.Scheme) error {
-	for _, operand := range sspOperands {
-		err := operand.AddWatchTypesToScheme(scheme)
-		if err != nil {
-			return err
-		}
-	}
-	return nil
-}
