@@ -8,24 +8,21 @@ import (
 	. "github.com/onsi/ginkgo"
 	. "github.com/onsi/gomega"
 	templatev1 "github.com/openshift/api/template/v1"
+	libhandler "github.com/operator-framework/operator-lib/handler"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/types"
-	"k8s.io/client-go/kubernetes/scheme"
-	. "kubevirt.io/ssp-operator/internal/test-utils"
+	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/client/fake"
 	logf "sigs.k8s.io/controller-runtime/pkg/log"
 	"sigs.k8s.io/controller-runtime/pkg/reconcile"
 
-	libhandler "github.com/operator-framework/operator-lib/handler"
 	ssp "kubevirt.io/ssp-operator/api/v1beta1"
 	"kubevirt.io/ssp-operator/internal/common"
-	"sigs.k8s.io/controller-runtime/pkg/client"
+	"kubevirt.io/ssp-operator/internal/operands"
+	. "kubevirt.io/ssp-operator/internal/test-utils"
 )
 
-var (
-	log     = logf.Log.WithName("common-templates-operand")
-	operand = GetOperand()
-)
+var log = logf.Log.WithName("common-templates-operand")
 
 const (
 	namespace = "kubevirt"
@@ -43,14 +40,17 @@ func TestTemplates(t *testing.T) {
 
 var _ = Describe("Common-Templates operand", func() {
 
-	var request common.Request
+	var (
+		testTemplates []templatev1.Template
+		operand       operands.Operand
+		request       common.Request
+	)
 
 	BeforeEach(func() {
-		s := scheme.Scheme
-		Expect(ssp.AddToScheme(s)).ToNot(HaveOccurred())
-		Expect(operand.AddWatchTypesToScheme(s)).ToNot(HaveOccurred())
+		testTemplates = getTestTemplates()
+		operand = New(testTemplates)
 
-		client := fake.NewFakeClientWithScheme(s)
+		client := fake.NewFakeClientWithScheme(common.Scheme)
 		request = common.Request{
 			Request: reconcile.Request{
 				NamespacedName: types.NamespacedName{
@@ -80,35 +80,14 @@ var _ = Describe("Common-Templates operand", func() {
 		}
 	})
 
-	It("should create golden-images namespace", func() {
-		_, err := operand.Reconcile(&request)
-		Expect(err).ToNot(HaveOccurred())
-		ExpectResourceExists(newGoldenImagesNS(GoldenImagesNSname), request)
-	})
 	It("should create common-template resources", func() {
 		_, err := operand.Reconcile(&request)
 		Expect(err).ToNot(HaveOccurred())
-		Expect(templatesBundle).ToNot(BeNil())
-		for _, template := range templatesBundle {
+		for _, template := range testTemplates {
 			template.Namespace = namespace
 			ExpectResourceExists(&template, request)
 		}
 	})
-	It("should create view role", func() {
-		_, err := operand.Reconcile(&request)
-		Expect(err).ToNot(HaveOccurred())
-		ExpectResourceExists(newViewRole(GoldenImagesNSname), request)
-	})
-	It("should create view role binding", func() {
-		_, err := operand.Reconcile(&request)
-		Expect(err).ToNot(HaveOccurred())
-		ExpectResourceExists(newViewRoleBinding(GoldenImagesNSname), request)
-	})
-	It("should create view role binding", func() {
-		_, err := operand.Reconcile(&request)
-		Expect(err).ToNot(HaveOccurred())
-		ExpectResourceExists(newEditRole(), request)
-	})
 
 	Context("old templates", func() {
 		var (
@@ -136,7 +115,7 @@ var _ = Describe("Common-Templates operand", func() {
 					Namespace: request.Instance.Spec.CommonTemplates.Namespace,
 					Labels: map[string]string{
 						TemplateVersionLabel: "not-latest",
-						TemplateTypeLabel:    "base",
+						TemplateTypeLabel:    TemplateTypeLabelBaseValue,
 						testOsLabel:          "true",
 						testFlavorLabel:      "true",
 						testWorkflowLabel:    "true",
@@ -188,7 +167,7 @@ var _ = Describe("Common-Templates operand", func() {
 			Expect(updatedTpl.Labels[testOsLabel]).To(Equal(""), TemplateOsLabelPrefix+" should be empty")
 			Expect(updatedTpl.Labels[testFlavorLabel]).To(Equal(""), TemplateFlavorLabelPrefix+" should be empty")
 			Expect(updatedTpl.Labels[testWorkflowLabel]).To(Equal(""), TemplateWorkloadLabelPrefix+" should be empty")
-			Expect(updatedTpl.Labels[TemplateTypeLabel]).To(Equal("base"), TemplateTypeLabel+" should equal base")
+			Expect(updatedTpl.Labels[TemplateTypeLabel]).To(Equal(TemplateTypeLabelBaseValue), TemplateTypeLabel+" should equal base")
 			Expect(updatedTpl.Labels[TemplateVersionLabel]).To(Equal("not-latest"), TemplateVersionLabel+" should equal not-latest")
 			Expect(updatedTpl.Annotations[TemplateDeprecatedAnnotation]).To(Equal("true"), TemplateDeprecatedAnnotation+" should not be empty")
 		})
@@ -210,7 +189,7 @@ var _ = Describe("Common-Templates operand", func() {
 					if strings.HasPrefix(label, TemplateWorkloadLabelPrefix) {
 						Expect(template.Labels[label]).To(Equal("true"), TemplateWorkloadLabelPrefix+" should not be empty")
 					}
-					Expect(template.Labels[TemplateTypeLabel]).To(Equal("base"), TemplateTypeLabel+" should equal base")
+					Expect(template.Labels[TemplateTypeLabel]).To(Equal(TemplateTypeLabelBaseValue), TemplateTypeLabel+" should equal base")
 					Expect(template.Labels[TemplateVersionLabel]).To(Equal(Version), TemplateVersionLabel+" should equal "+Version)
 				}
 				Expect(template.Annotations[TemplateDeprecatedAnnotation]).To(Equal(""), TemplateDeprecatedAnnotation+" should be empty")
@@ -219,3 +198,29 @@ var _ = Describe("Common-Templates operand", func() {
 		})
 	})
 })
+
+func getTestTemplates() []templatev1.Template {
+	return []templatev1.Template{{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: "centos8-server-medium",
+			Labels: map[string]string{
+				TemplateTypeLabel:                      TemplateTypeLabelBaseValue,
+				TemplateVersionLabel:                   "v0.16.2",
+				TemplateOsLabelPrefix + "centos8":      "true",
+				TemplateFlavorLabelPrefix + "medium":   "true",
+				TemplateWorkloadLabelPrefix + "server": "true",
+			},
+		},
+	}, {
+		ObjectMeta: metav1.ObjectMeta{
+			Name: "windows10-desktop-medium",
+			Labels: map[string]string{
+				TemplateTypeLabel:                       TemplateTypeLabelBaseValue,
+				TemplateVersionLabel:                    "v0.16.2",
+				TemplateOsLabelPrefix + "win10":         "true",
+				TemplateFlavorLabelPrefix + "medium":    "true",
+				TemplateWorkloadLabelPrefix + "desktop": "true",
+			},
+		},
+	}}
+}
